import React, { useContext, useState, useRef, useEffect } from 'react';
import ReactDOM from 'react-dom'
import ReactFlow, { useReactFlow } from 'reactflow'
import { pillWidth, pillHeight, pillXPadding, pillInnerMargin, topLineHeight, pillBorderWidth, pillAttributesPadding } from './pill.const';
import { Position } from 'reactflow';
import { PillHandle } from './PillHandle';
import { PillContext } from './PillContext';
import { QueryRelationDirection } from 'ts-common';

export type PillI = {
  onHovered?: (hovered: boolean) => void;
  // onDragged: (dragged: boolean) => void;
  title: string | React.ReactNode;
  children?: React.ReactNode;
  handles?: React.ReactNode;
  style?: React.CSSProperties;
  corner?: 'rounded' | 'square' | 'diamond' | 'diamond-inner' | 'diamond-left' | 'diamond-right';
  direction?: QueryRelationDirection;
  handleUp?: React.ReactNode;
  handleDown?: React.ReactNode;
  handleLeft?: React.ReactNode;
  handleRight?: React.ReactNode;
  className?: string;
  draggable?: boolean;
  width?: number;
  height?: number;
};

export const Pill = React.memo((props: PillI) => {
  const pillContext = useContext(PillContext);
  const [hovered, setHovered] = useState(false);

  const onMouseEnter = (event: React.MouseEvent) => {
    if (!hovered) {
      setHovered(true);
      if (props.onHovered) props.onHovered(true);
    }
  };

  const onMouseLeave = (event: React.MouseEvent) => {
    if (hovered) {
      setHovered(false);
      if (props.onHovered) props.onHovered(false);
    }
  };

  const width = pillWidth;
  const corner = props.corner || 'rounded';

  const innerContentStyle = {
    minWidth: corner === 'diamond' ? width - pillBorderWidth * 2 : width - pillBorderWidth * 2,
    maxWidth: corner === 'diamond' ? width - pillBorderWidth * 2 : width - pillBorderWidth * 2,
    minHeight: pillHeight - pillBorderWidth * 2,
    maxHeight: pillHeight - pillBorderWidth * 2,
    margin: pillBorderWidth,
    clipPath: 'none',
  };

  const outerContentStyle = {
    minWidth: width,
    maxWidth: width,
    minHeight: pillHeight,
    maxHeight: pillHeight,
    clipPath: 'none',
  };

  if (corner === 'diamond') {
    innerContentStyle.clipPath = 'polygon(0.5% 50%, 4.9% 0%, 95.1% 0%, 99.5% 50%, 95.1% 100%, 4.9% 100%)';
    outerContentStyle.clipPath = 'polygon(0% 50%, 5% 0%, 95% 0%, 100% 50%, 95% 100%, 5% 100%)';
  } else if (corner === 'diamond-inner') {
    innerContentStyle.clipPath = 'polygon(5% 50%, 0% 0%, 100% 0%, 95% 50%, 100% 100%, 0% 100%)';
    outerContentStyle.clipPath = 'polygon(5% 50%, 0% 0%, 100% 0%, 95% 50%, 100% 100%, 0% 100%)';
  } else if (corner === 'diamond-left') {
    innerContentStyle.clipPath = 'polygon(0.5% 50%, 4.9% 0%, 100% 0%, 95% 50%, 100% 100%, 0% 100%)';
    outerContentStyle.clipPath = 'polygon(0% 50%, 5% 0%, 100% 0%, 95% 50%, 100% 100%, 5% 100%)';
  } else if (corner === 'diamond-right') {
    innerContentStyle.clipPath = 'polygon(5% 50%, 0% 0%, 95.1% 0%, 99.5% 50%, 95.1% 100%, 0% 100%)';
    outerContentStyle.clipPath = 'polygon(5% 50%, 0% 0%, 95% 0%, 100% 50%, 95% 100%, 0% 100%)';
  }

  const onDragStart = (event: React.DragEvent<HTMLDivElement>) => {
    const pill = event.target as HTMLDivElement;
    const dragImage = pill.cloneNode(true) as HTMLDivElement;
    dragImage.style.transform = 'translate(0, 0)'; // Removes the background from drag image in Chrome
    document.body.appendChild(dragImage);

    const mouse_x = event.clientX - pill.getBoundingClientRect().left;
    const mouse_y = event.clientY - pill.getBoundingClientRect().top;

    event.dataTransfer.setDragImage(dragImage, mouse_x, mouse_y);
    event.dataTransfer.setData('mouse_x', String(mouse_x));
    event.dataTransfer.setData('mouse_y', String(mouse_y));

    setTimeout(() => {
      dragImage.remove();
    }, 1);
  };

  return (
    <div
      draggable={props.draggable || false}
      onDragStart={props.draggable ? onDragStart : undefined}
      className={(props.className ? props.className + ' ' : '') + 'flex flex-row flex-grow-0 text-xs text-justify relative z-10'}
    >
      <div
        className={'bg-secondary-200 ' + (corner !== 'square' ? 'rounded' : '')}
        style={{
          ...outerContentStyle,
        }}
      >
        <div
          className={'bg-secondary-100 ' + (corner !== 'square' ? 'rounded-[3px]' : '')}
          style={{
            ...innerContentStyle,
          }}
        >
          {pillContext.color && (
            <div
              className={corner !== 'square' ? 'rounded-t ' : ''}
              style={{ height: topLineHeight, backgroundColor: pillContext.color }}
            ></div>
          )}
          <div
            className={'font-semibold ' + (corner !== 'square' ? 'rounded-b-[3px]' : '')}
            onMouseEnter={onMouseEnter}
            onMouseLeave={onMouseLeave}
          >
            <div
              className=""
              style={{
                transform: `translateY(${topLineHeight / 3}px)`,
                paddingLeft: pillXPadding + (corner === 'diamond' ? 5 : 0),
                paddingRight: pillXPadding + (corner === 'diamond' ? 5 : 0),
              }}
            >
              {props.title}
            </div>
          </div>
        </div>
      </div>

      <div
        className="absolute -z-10"
        style={{
          top: pillHeight - pillInnerMargin + (corner === 'diamond' ? pillHeight / 2 - pillBorderWidth : 0),
          width: pillWidth - (corner === 'diamond' ? pillWidth * 0.05 : 0),
          paddingLeft: pillAttributesPadding + (corner === 'diamond' ? pillWidth * 0.05 : 0),
          paddingRight: pillAttributesPadding,
        }}
      >
        {props.children}
      </div>
      <div className="absolute z-50 pointer-events-auto bg-black">{props.handles}</div>
    </div>
  );
});

export const EntityPill = React.memo((props: Omit<PillI, 'topColor'> & { withHandles?: 'vertical' | 'horizontal' }) => {
  const handles = !props.withHandles ? undefined : props.withHandles === 'horizontal' ? (
    <>
      <PillHandle position={Position.Left} className={'stroke-secondary-200'} type="square">
        {props.handleLeft}
      </PillHandle>
      <PillHandle position={Position.Right} className={'stroke-secondary-200'} type="square">
        {props.handleRight}
      </PillHandle>
    </>
  ) : (
    <>
      <PillHandle position={Position.Top} className={'stroke-secondary-200 stroke-1'} type="arrowUp">
        {props.handleUp}
      </PillHandle>
      <PillHandle position={Position.Bottom} className={'stroke-secondary-200 stroke-1'} type="arrowDown">
        {props.handleDown}
      </PillHandle>
    </>
  );
  return (
    <PillContext.Provider value={{ color: 'hsl(var(--clr-node))' }}>
      <Pill {...props} corner="rounded" handles={handles} className="relative z-10" />
    </PillContext.Provider>
  );
});




export const QueryMetaPill = React.memo((node: SchemaReactflowMetaNode) => {
  const ref = useRef<HTMLDivElement | null>(null);
  const data = node.data;
  console.log("InsideQueryMetapill", data)

  return (
    <div className="w-fit h-fit nowheel" ref={ref}>
      <MetaPill
        width={data?.width}
        selection={data?.selection}
        height={data?.height}
        title={undefined}
        {...data?.attributes && (
          <PillAttributes
            node={node}
            attributes={[]}
            attributeEdges={[]}
          />
        )}
      />
    </div>
  );
});


export const MetaPill = React.memo(
  ({
    width = 400,
    height = 200,
    style,
    setDragging,
    selection = [],
    setSelection, // Added setter for selection here
    ...props
  }: {
    width?: number;
    height?: number;
    setDragging?: (dragging: boolean) => void;
    selection?: any[];
    setSelection?: (selection: any[]) => void; // Setter for selection to update state
  }) => {
    const [isMenuVisible, setIsMenuVisible] = useState(false);
    const [size, setSize] = useState({ width, height });
    const [isHovered, setIsHovered] = useState(false);

    useEffect(() => {
      setSize({ width, height });
    }, [width, height]);

    const resizingRef = useRef(false);

    const handleClick = (event: React.MouseEvent) => {
      event.preventDefault();
      setIsMenuVisible(true);
    };

    const handleClose = () => {
      setIsMenuVisible(false);
    };

    const handleMouseDown = (event: React.MouseEvent) => {
      event.preventDefault();
      event.stopPropagation();
      resizingRef.current = true;
      setDragging && setDragging(false); // Disable React Flow drag
      document.addEventListener("mousemove", handleMouseMove);
      document.addEventListener("mouseup", handleMouseUp);
    };

    const handleMouseMove = (event: MouseEvent) => {
      if (!resizingRef.current) return;
      setSize((prevSize) => ({
        width: Math.max(100, prevSize.width + event.movementX),
        height: Math.max(100, prevSize.height + event.movementY),
      }));
    };

    const handleMouseUp = () => {
      resizingRef.current = false;
      setDragging && setDragging(true); // Re-enable React Flow drag
      document.removeEventListener("mousemove", handleMouseMove);
      document.removeEventListener("mouseup", handleMouseUp);
    };

    const handleDragEnter = (event: React.DragEvent) => {
      event.preventDefault();
      setIsHovered(true);
    };

    const handleDragLeave = (event: React.DragEvent) => {
      event.preventDefault();
      setIsHovered(false);
    };

    const handleDrop = (event: React.DragEvent) => {
      event.preventDefault();
      setIsHovered(false);
      console.log("DROP");
      const draggedItem = event.dataTransfer.getData("application/reactflow");
      console.log("dragged item", draggedItem)
      console.log("set selection", setSelection)

      if (draggedItem  && setSelection) {

        console.log("set selection", setSelection)
        const parsedItem = JSON.parse(draggedItem);
        console.log("parseditem ", parsedItem)
        setSelection([...selection, parsedItem]); // Update selection state
      }
    };

    return (
      <>
        <div
          className="meta-pill flex items-end justify-end rounded-lg relative"
          onDragEnter={handleDragEnter}
          onDragLeave={handleDragLeave}
          onDragOver={(e) => e.preventDefault()}
          onDrop={handleDrop}
          style={{
            width: size.width,
            height: size.height,
            backgroundColor: isHovered
              ? "rgba(255, 165, 0, 0.3)"
              : "rgba(255, 165, 0, 0.5)", // Lighter color on hover
            position: "relative",
            zIndex: 1,
            pointerEvents: resizingRef.current ? "none" : "auto",
            ...style,
          }}
        >
          <button
            onClick={handleClick}
            style={{
              padding: "10px 20px",
              backgroundColor: "#FF6F00",
              borderRadius: "8px",
              border: "none",
              color: "white",
              cursor: "pointer",
              display: "block",
            }}
          >
            Open Menu
          </button>

          <div
            className="nodrag"
            onMouseDown={handleMouseDown}
            style={{
              position: "absolute",
              bottom: 0,
              right: 0,
              width: "16px",
              height: "16px",
              backgroundColor: "gray",
              cursor: "se-resize",
            }}
          />

          {/* Selection Count in Lower Left Corner */}
          <div
            style={{
              position: "absolute",
              bottom: "4px",
              left: "4px",
              backgroundColor: "rgba(0, 0, 0, 0.6)",
              color: "white",
              padding: "2px 6px",
              borderRadius: "4px",
              fontSize: "12px",
            }}
          >
            {selection.length} selected
          </div>
        </div>

        {isMenuVisible &&
          ReactDOM.createPortal(
            <div
              style={{
                position: "fixed",
                top: "50%",
                left: "50%",
                transform: "translate(-50%, -50%)",
                width: "50vw",
                height: "50vh",
                backgroundColor: "white",
                color: "black",
                border: "1px solid #ccc",
                padding: "20px",
                zIndex: 9999,
                boxShadow: "0 4px 10px rgba(0,0,0,0.2)",
                display: "flex",
                flexDirection: "column",
                justifyContent: "center",
                alignItems: "center",
              }}
            >
              <h3>Choose filters:</h3>
              <ul>
                <li>Filter 1</li>
                <li>Filter 2</li>
                <li>Filter 3</li>
              </ul>
              <button
                onClick={handleClose}
                style={{ marginTop: "20px", padding: "10px", cursor: "pointer" }}
              >
                Close
              </button>
            </div>,
            document.body
          )}
      </>
    );
  }
);
export const RelationPill = React.memo((props: Omit<PillI, 'topColor'> & { withHandles?: 'vertical' | 'horizontal' }) => {
  const handles = !props.withHandles ? undefined : props.withHandles === 'horizontal' ? (
    <>
      <PillHandle position={Position.Left} className={'stroke-secondary-200'} type="square">
        {props.handleLeft}
      </PillHandle>
      <PillHandle position={Position.Right} className={'stroke-secondary-200'} type="square" mr={-pillBorderWidth}>
        {props.handleRight}
      </PillHandle>
    </>
  ) : (
    <>
      <PillHandle position={Position.Top} className={'stroke-secondary-200 stroke-1'} type="arrowUp">
        {props.handleUp}
      </PillHandle>
      <PillHandle position={Position.Bottom} className={'stroke-secondary-200 stroke-1'} type="arrowDown">
        {props.handleDown}
      </PillHandle>
    </>
  );

  return (
    <PillContext.Provider value={{ color: 'hsl(var(--clr-relation))' }}>
      <Pill
        {...props}
        corner={props.direction === 'Left' ? 'diamond-left' : props.direction === 'Right' ? 'diamond-right' : 'diamond'}
        handles={handles}
      />
    </PillContext.Provider>
  );
});

export const LogicPill = React.memo((props: Omit<PillI, 'topColor'>) => {
  const handles = (
    <PillHandle position={Position.Left} className={'stroke-white'} type="square">
      {props.handleLeft}
    </PillHandle>
  );

  return (
    <PillContext.Provider value={{ color: 'hsl(var(--clr-filter))' }}>
      <Pill {...props} corner="square" handles={handles} />
    </PillContext.Provider>
  );
});
